/* Author: Ruud Harmsen, October-November 2023

   See https://rudhar.com/musica/hwedwortehun/ia13.htm for explanations.
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "../../rhmusicfuncs/rhmusicfuncs.h"
#include "../../rhmusicfuncs/rhmusicadhoc.h"

static int Chord[13];
#define NotesInChord ((int)(sizeof Chord / sizeof Chord[0]))
static struct DegreeData wrkC[NotesInChord];
static struct DegreeData resC[NotesInChord];

static void GetChordFromCL (int argc, char **argv);
static int  GetChordFromStdin (void);
static int  compar (const void *a, const void *b);

/***********************************************************************/
int main (int argc, char **argv)
{
   int verbose = 1;

   if (argc > 1)
   {
      if (!strcmp(argv[1], "-v"))
         verbose = 2;
      else if (!strcmp(argv[1], "-vv"))
         verbose = 3;
      else if (!strcmp(argv[1], "-vvv"))
         verbose = 4;
      argc--, argv++;
   }

   if (argc > 1)
   {
      GetChordFromCL(argc, argv);
      AdhocRunIt(Chord, NotesInChord, wrkC, resC, verbose);
   }
   else
   {
      while (GetChordFromStdin())
         AdhocRunIt(Chord, NotesInChord, wrkC, resC, verbose);
   }
   return 0;
}

/***********************************************************************/
static void GetChordFromCL (int argc, char **argv)
{
   int i;

   for (i = 0; i < NotesInChord && argc > 1; i++, argv++, argc--)
   {
      Chord[i] = (int)(atof(argv[1]) * 2.0 + 0.5);
      if (Chord[i] >= NumIntvals)
          Chord[i] = 1000;
   }
   for (; i < NotesInChord; i++)
      Chord[i] = 1000;

   qsort(Chord, NotesInChord, sizeof Chord[0], compar);
}
/***********************************************************************/
static int GetChordFromStdin (void)
{
   char line[128], *pl = line;
   int i;

   if (!fgets(line, sizeof line, stdin) || line[0] == 'q')
      return 0;

   for (i = 0; i < NotesInChord; i++)
   {
      int consumed, ret;
      float scanned;

      ret = sscanf(pl, "%f%n", &scanned, &consumed);
      if (ret < 1)
         break;

      pl += consumed;
      Chord[i] = (int)(scanned * 2.0 + 0.5);

      if (Chord[i] >= NumIntvals)
          Chord[i] = 1000;
   }
   for (; i < NotesInChord; i++)
      Chord[i] = 1000;

   qsort(Chord, NotesInChord, sizeof Chord[0], compar);

   return 1;
}
/***********************************************************************/
int compar(const void *a, const void *b)
{
   return *(int *)a - *(int *)b;
}
